@@ -39,6 +39,7 @@
 #include <QWaitCondition>
 #include <QCryptographicHash>
 #include <QFile>
+#include <QTimer>
 
 #include "nm-openconnect-service.h"
 
@@ -72,6 +73,7 @@ public:
     OpenconnectAuthWorkerThread *worker;
     QList<VPNHost> hosts;
     bool userQuit;
+    bool formGroupChanged;
     int cancelPipes[2];
     QList<QPair<QString, int> > serverLog;
 
@@ -86,6 +88,8 @@ OpenconnectAuthWidget::OpenconnectAuthWidget(const NetworkManager::VpnSetting::P
     d->setting = setting;
     d->ui.setupUi(this);
     d->userQuit = false;
+    d->formGroupChanged = false;
+
     if (pipe2(d->cancelPipes, O_NONBLOCK|O_CLOEXEC)) {
         // Should never happen. Just don't do real cancellation if it does
         d->cancelPipes[0] = -1;
@@ -95,13 +99,12 @@ OpenconnectAuthWidget::OpenconnectAuthWidget(const NetworkManager::VpnSetting::P
     connect(d->ui.cmbLogLevel, SIGNAL(currentIndexChanged(int)), this, SLOT(logLevelChanged(int)));
     connect(d->ui.viewServerLog, SIGNAL(toggled(bool)), this, SLOT(viewServerLogToggled(bool)));
     connect(d->ui.btnConnect, SIGNAL(clicked()), this, SLOT(connectHost()));
-    connect(d->ui.cmbHosts, SIGNAL(currentIndexChanged(int)), this, SLOT(connectHost()));
 
     d->ui.cmbLogLevel->setCurrentIndex(OpenconnectAuthWidgetPrivate::Debug);
     d->ui.btnConnect->setIcon(KIcon("network-connect"));
     d->ui.viewServerLog->setChecked(false);
 
-    d->worker = new OpenconnectAuthWorkerThread(&d->mutex, &d->workerWaiting, &d->userQuit, d->cancelPipes[0]);
+    d->worker = new OpenconnectAuthWorkerThread(&d->mutex, &d->workerWaiting, &d->userQuit, &d->formGroupChanged, d->cancelPipes[0]);
 
     // gets the pointer to struct openconnect_info (defined in openconnect.h), which contains data that OpenConnect needs,
     // and which needs to be populated with settings we get from NM, like host, certificate or private key
@@ -116,6 +119,9 @@ OpenconnectAuthWidget::OpenconnectAuthWidget(const NetworkManager::VpnSetting::P
     readConfig();
     readSecrets();
 
+    // This might be set by readSecrets() so don't connect it until now
+    connect(d->ui.cmbHosts, SIGNAL(currentIndexChanged(int)), this, SLOT(connectHost()));
+
     KAcceleratorManager::manage(this);
 }
 
@@ -228,8 +234,9 @@ void OpenconnectAuthWidget::readSecrets()
 
     if (d->secrets["autoconnect"] == "yes") {
         d->ui.chkAutoconnect->setChecked(true);
-        connectHost();
+        QTimer::singleShot(0, this, SLOT(connectHost()));
     }
+
     if (!d->secrets["certsigs"].isEmpty()) {
         d->certificateFingerprints.append(d->secrets["certsigs"].split('\t'));
     }
@@ -414,7 +421,7 @@ void OpenconnectAuthWidget::processAuthForm(struct oc_auth_form *form)
     int passwordnumber = 0;
     bool focusSet = false;
     for (opt = form->opts; opt; opt = opt->next) {
-        if (opt->type == OC_FORM_OPT_HIDDEN)
+        if (opt->type == OC_FORM_OPT_HIDDEN || IGNORE_OPT(opt))
             continue;
         QLabel *text = new QLabel(this);
         text->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
@@ -441,9 +448,18 @@ void OpenconnectAuthWidget::processAuthForm(struct oc_auth_form *form)
             KComboBox *cmb = new KComboBox(this);
             struct oc_form_opt_select *sopt = reinterpret_cast<oc_form_opt_select *>(opt);
             for (int i = 0; i < sopt->nr_choices; i++) {
-                cmb->addItem(QString::fromUtf8(sopt->choices[i].label), QString::fromUtf8(sopt->choices[i].name));
-                if (value == QString::fromUtf8(sopt->choices[i].name))
+                cmb->addItem(QString::fromUtf8(FORMCHOICE(sopt, i)->label),
+                             QString::fromUtf8(FORMCHOICE(sopt, i)->name));
+                if (value == QString::fromUtf8(FORMCHOICE(sopt, i)->name)) {
                     cmb->setCurrentIndex(i);
+                    if (sopt == AUTHGROUP_OPT(form) &&
+                        i != AUTHGROUP_SELECTION(form)) {
+                        QTimer::singleShot(0, this, SLOT(formGroupChanged()));
+                    }
+                }
+            }
+            if (sopt == AUTHGROUP_OPT(form)) {
+                connect(cmb, SIGNAL(currentIndexChanged(int)), this, SLOT(formGroupChanged()));
             }
             widget = qobject_cast<QWidget*>(cmb);
         }
@@ -534,12 +550,21 @@ void OpenconnectAuthWidget::validatePeerCert(const QString &fingerprint,
     d->mutex.unlock();
 }
 
+void OpenconnectAuthWidget::formGroupChanged()
+{
+    Q_D(OpenconnectAuthWidget);
+
+    d->formGroupChanged = true;
+    formLoginClicked();
+}
+
 // Writes the user input from the form into the oc_auth_form structs we got from
 // libopenconnect, and wakes the worker thread up to try to log in and obtain a
 // cookie with this data
 void OpenconnectAuthWidget::formLoginClicked()
 {
     Q_D(OpenconnectAuthWidget);
+
     const int lastIndex = d->ui.loginBoxLayout->count() - 1;
     QLayout *layout = d->ui.loginBoxLayout->itemAt(lastIndex - 2)->layout();
     struct oc_auth_form *form = (struct oc_auth_form *) d->ui.loginBoxLayout->itemAt(lastIndex)->widget()->property("openconnect_form").value<quintptr>();
