@@ -40,6 +40,25 @@ struct x509_st;
 #define OPENCONNECT_OPENSSL
 #endif
 
+#if OPENCONNECT_CHECK_VER(3,0)
+#define NEWGROUP_SUPPORTED	1
+#define AUTHGROUP_OPT(form)	(void *)(form)->authgroup_opt
+#define AUTHGROUP_SELECTION(form) (form)->authgroup_selection
+#define FORMCHOICE(sopt, i)	((sopt)->choices[i])
+#define IGNORE_OPT(opt)		((opt)->flags & OC_FORM_OPT_IGNORE)
+#else
+#define NEWGROUP_SUPPORTED	0
+#define AUTHGROUP_OPT(form)	NULL
+#define AUTHGROUP_SELECTION(form) 0
+#define FORMCHOICE(sopt, i)	(&(sopt)->choices[i])
+#define IGNORE_OPT(opt)		0
+
+#define OC_FORM_RESULT_ERR	-1
+#define OC_FORM_RESULT_OK	0
+#define OC_FORM_RESULT_CANCELLED 1
+#define OC_FORM_RESULT_NEWGROUP	2
+#endif
+
 #include <QThread>
 
 class QMutex;
@@ -51,7 +70,7 @@ class OpenconnectAuthWorkerThread : public QThread
     Q_OBJECT
     friend class OpenconnectAuthStaticWrapper;
 public:
-    OpenconnectAuthWorkerThread(QMutex *, QWaitCondition *, bool *, int);
+    OpenconnectAuthWorkerThread(QMutex *, QWaitCondition *, bool *, bool *, int);
     ~OpenconnectAuthWorkerThread();
     struct openconnect_info* getOpenconnectInfo();
 
@@ -74,6 +93,7 @@ private:
     QMutex *m_mutex;
     QWaitCondition *m_waitForUserInput;
     bool *m_userDecidedToQuit;
+    bool *m_formGroupChanged;
     struct openconnect_info *m_openconnectInfo;
 };
 
