@@ -1,6 +1,8 @@
 /*
     Copyright 2008 Will Stephenson <wstephenson@kde.org>
     Copyright 2011-2012 Rajeesh K Nambiar <rajeeshknambiar@gmail.com>
+    Copyright 2011 Ilia Kats <ilia-kats@gmx.net>
+    Copyright 2012-2014 Lamarque V. Souza <lamarque@kde.org>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -24,6 +26,7 @@
 #include <QStringBuilder>
 #include <KPluginFactory>
 #include <KMessageBox>
+#include <KStandardDirs>
 
 #include <NetworkManagerQt/Connection>
 #include <NetworkManagerQt/VpnSetting>
@@ -34,6 +37,9 @@
 
 #include "nm-openvpn-service.h"
 
+//#include <nm-setting-ip4-config.h>
+#define NM_SETTING_IP4_CONFIG_NEVER_DEFAULT "never-default"
+
 K_PLUGIN_FACTORY(OpenVpnUiPluginFactory, registerPlugin<OpenVpnUiPlugin>();)
 K_EXPORT_PLUGIN(OpenVpnUiPluginFactory("plasmanetworkmanagement_openvpnui"))
 
@@ -64,6 +70,19 @@ K_EXPORT_PLUGIN(OpenVpnUiPluginFactory("plasmanetworkmanagement_openvpnui"))
 #define TLS_CLIENT_TAG "tls-client"
 #define TLS_REMOTE_TAG "tls-remote"
 #define TUNMTU_TAG "tun-mtu"
+#define REDIRECT_GATEWAY_TAG "redirect-gateway"
+#define KEY_DIRECTION_TAG "key-direction"
+
+#define BEGIN_KEY_CA_TAG "<ca>"
+#define END_KEY_CA_TAG "</ca>"
+#define BEGIN_KEY_CERT_TAG "<cert>"
+#define END_KEY_CERT_TAG "</cert>"
+#define BEGIN_KEY_KEY_TAG "<key>"
+#define END_KEY_KEY_TAG "</key>"
+#define BEGIN_KEY_SECRET_TAG "<secret>"
+#define END_KEY_SECRET_TAG "</secret>"
+#define BEGIN_TLS_AUTH_TAG "<tls-auth>"
+#define END_TLS_AUTH_TAG "</tls_auth>"
 
 #define PROC_TYPE_TAG "Proc-Type: 4,ENCRYPTED"
 #define PKCS8_TAG "-----BEGIN ENCRYPTED PRIVATE KEY-----"
@@ -166,6 +185,7 @@ NMVariantMapMap OpenVpnUiPlugin::importConnectionSettings(const QString &fileNam
         return result;
     }
 
+    const QString connectionName = QFileInfo(fileName).completeBaseName();
     NMStringMap dataMap;
     NMStringMap secretData;
     QVariantMap ipv4Data;
@@ -178,6 +198,8 @@ NMVariantMapMap OpenVpnUiPlugin::importConnectionSettings(const QString &fileNam
     bool proxy_set = false;
     bool have_pass = false;
     bool have_sk = false;
+    bool have_redirect_gateway = false;
+    int key_direction = -1;
 
     QTextStream in(&impFile);
     while (!in.atEnd()) {
@@ -369,22 +391,26 @@ NMVariantMapMap OpenVpnUiPlugin::importConnectionSettings(const QString &fileNam
         }
         if (key_value[0] == CA_TAG && key_value.count() > 1) {
             key_value[1] = line.right(line.length() - line.indexOf(QRegExp("\\s"))); // Get whole string after key
-            dataMap.insert(QLatin1String(NM_OPENVPN_KEY_CA), unQuote(key_value[1], fileName));
+            const QString absoluteFilePath = tryToCopyToCertificatesDirectory(connectionName, unQuote(key_value[1], fileName));
+            dataMap.insert(QLatin1String(NM_OPENVPN_KEY_CA), absoluteFilePath);
             continue;
         }
         if (key_value[0] == CERT_TAG && key_value.count() > 1) {
             key_value[1] = line.right(line.length() - line.indexOf(QRegExp("\\s"))); // Get whole string after key
-            dataMap.insert(QLatin1String(NM_OPENVPN_KEY_CERT), unQuote(key_value[1], fileName));
+            const QString absoluteFilePath = tryToCopyToCertificatesDirectory(connectionName, unQuote(key_value[1], fileName));
+            dataMap.insert(QLatin1String(NM_OPENVPN_KEY_CERT), absoluteFilePath);
             continue;
         }
         if (key_value[0] == KEY_TAG && key_value.count() > 1) {
             key_value[1] = line.right(line.length() - line.indexOf(QRegExp("\\s"))); // Get whole string after key
-            dataMap.insert(QLatin1String(NM_OPENVPN_KEY_KEY), unQuote(key_value[1], fileName));
+            const QString absoluteFilePath = tryToCopyToCertificatesDirectory(connectionName, unQuote(key_value[1], fileName));
+            dataMap.insert(QLatin1String(NM_OPENVPN_KEY_KEY), absoluteFilePath);
             continue;
         }
         if (key_value[0] == SECRET_TAG && key_value.count() > 1) {
             key_value[1] = line.right(line.length() - line.indexOf(QRegExp("\\s"))); // Get whole string after key
-            dataMap.insert(QLatin1String(NM_OPENVPN_KEY_STATIC_KEY), unQuote(key_value[1], fileName));
+            const QString absoluteFilePath = tryToCopyToCertificatesDirectory(connectionName, unQuote(key_value[1], fileName));
+            dataMap.insert(QLatin1String(NM_OPENVPN_KEY_STATIC_KEY), absoluteFilePath);
             if (key_value.count() > 2) {
                 key_value[2] = key_value[1];
                 if (!key_value[2].isEmpty() && (key_value[2].toLong() == 0 ||key_value[2].toLong() == 1))
@@ -395,7 +421,8 @@ NMVariantMapMap OpenVpnUiPlugin::importConnectionSettings(const QString &fileNam
         }
         if (key_value[0] == TLS_AUTH_TAG && key_value.count() >1) {
             key_value[1] = line.right(line.length() - line.indexOf(QRegExp("\\s"))); // Get whole string after key
-            dataMap.insert(QLatin1String(NM_OPENVPN_KEY_TA), unQuote(key_value[1], fileName));
+            const QString absoluteFilePath = tryToCopyToCertificatesDirectory(connectionName, unQuote(key_value[1], fileName));
+            dataMap.insert(QLatin1String(NM_OPENVPN_KEY_TA), absoluteFilePath);
             if (key_value.count() > 2) {
                 key_value[2] = key_value[1];
                 if (!key_value[2].isEmpty() && (key_value[2].toLong() == 0 ||key_value[2].toLong() == 1))
@@ -443,12 +470,75 @@ NMVariantMapMap OpenVpnUiPlugin::importConnectionSettings(const QString &fileNam
             }
             continue;
         }
+        if (key_value[0] == REDIRECT_GATEWAY_TAG) {
+            have_redirect_gateway = true;
+            continue;
+        }
+        if (key_value[0] == KEY_DIRECTION_TAG) {
+            if (key_value.count() == 2) {
+                key_direction = key_value[1].toInt();
+            }
+
+            if (key_direction != 0 && key_direction != 1) {
+                KMessageBox::information(0, i18n("Invalid argument in option: %1", line));
+                key_direction = -1;
+            }
+            continue;
+        }
+
+        if (key_value[0] == BEGIN_KEY_CA_TAG) {
+            const QString caAbsolutePath = saveFile(in, QLatin1String(END_KEY_CA_TAG), connectionName, "ca.crt");
+            if (!caAbsolutePath.isEmpty()) {
+                dataMap.insert(QLatin1String(NM_OPENVPN_KEY_CA), caAbsolutePath);
+            }
+            continue;
+        } else if (key_value[0] == BEGIN_KEY_CERT_TAG) {
+            const QString certAbsolutePath = saveFile(in, QLatin1String(END_KEY_CERT_TAG), connectionName, "cert.crt");
+            if (!certAbsolutePath.isEmpty()) {
+                dataMap.insert(QLatin1String(NM_OPENVPN_KEY_CERT), certAbsolutePath);
+            }
+            continue;
+        } else if (key_value[0] == BEGIN_KEY_KEY_TAG) {
+            const QString keyAbsolutePath = saveFile(in, QLatin1String(END_KEY_KEY_TAG), connectionName, "private.key");
+            if (!keyAbsolutePath.isEmpty()) {
+                dataMap.insert(QLatin1String(NM_OPENVPN_KEY_KEY), keyAbsolutePath);
+            }
+            continue;
+        } else if (key_value[0] == BEGIN_KEY_SECRET_TAG) {
+            const QString secretAbsolutePath = saveFile(in, QLatin1String(END_KEY_SECRET_TAG), connectionName, "secret.key");
+            if (!secretAbsolutePath.isEmpty()) {
+                dataMap.insert(QLatin1String(NM_OPENVPN_KEY_KEY), secretAbsolutePath);
+                have_sk = true;
+
+                if (key_direction > -1) {
+                    dataMap.insert(QLatin1String(NM_OPENVPN_KEY_STATIC_KEY_DIRECTION), QString().setNum(key_direction));
+                }
+            }
+            continue;
+        } else if (key_value[0] == BEGIN_TLS_AUTH_TAG) {
+            const QString tlsAuthAbsolutePath = saveFile(in, QLatin1String(END_TLS_AUTH_TAG), connectionName, "tls_auth.key");
+            if (!tlsAuthAbsolutePath.isEmpty()) {
+                dataMap.insert(QLatin1String(NM_OPENVPN_KEY_TA), tlsAuthAbsolutePath);
+
+                if (key_direction > -1) {
+                    dataMap.insert(QLatin1String(NM_OPENVPN_KEY_TA_DIR), QString().setNum(key_direction));
+                }
+            }
+            continue;
+        }
+
         // Import X-NM-Routes if present
         if (key_value[0] == "X-NM-Routes") {
             ipv4Data.insert("X-NM-Routes", key_value[1]);
             continue;
         }
     }
+    // NetworkManager set vpn connections as default route by default. If the
+    // imported file does not contain "redirect-gateway" entry then set the
+    // connection as never default.
+    if (!have_redirect_gateway) {
+        ipv4Data.insert(NM_SETTING_IP4_CONFIG_NEVER_DEFAULT, "true");
+    }
     if (!have_client && !have_sk) {
         mError = VpnUiPlugin::Error;
         mErrorMessage = i18n("File %1 is not a valid OpenVPN's client configuration file", fileName);
@@ -500,7 +590,7 @@ NMVariantMapMap OpenVpnUiPlugin::importConnectionSettings(const QString &fileNam
     setting.setSecrets(secretData);
 
     QVariantMap conn;
-    conn.insert("id", QFileInfo(fileName).completeBaseName());
+    conn.insert("id", connectionName);
     conn.insert("type", "vpn");
     result.insert("connection", conn);
 
@@ -514,6 +604,48 @@ NMVariantMapMap OpenVpnUiPlugin::importConnectionSettings(const QString &fileNam
     return result;
 }
 
+QString OpenVpnUiPlugin::saveFile(QTextStream &in, const QString &endTag, const QString &connectionName, const QString &fileName)
+{
+    const QString certificatesDirectory = KStandardDirs::locateLocal("data", "networkmanagement/certificates/" + connectionName);
+    const QString absoluteFilePath = certificatesDirectory + '/' + fileName;
+    QFile outFile(absoluteFilePath);
+
+    QDir().mkpath(certificatesDirectory);
+    if (!outFile.open(QFile::WriteOnly | QFile::Text)) {
+        KMessageBox::information(0, i18n("Error saving file %1: %2", absoluteFilePath, outFile.errorString()));
+        return QString();
+    }
+
+    QTextStream out(&outFile);
+    while (!in.atEnd()) {
+        const QString line = in.readLine();
+
+        if (line.indexOf(endTag) >= 0) {
+            break;
+        }
+
+        out << line << "\n";
+    }
+
+    outFile.close();
+    return absoluteFilePath;
+}
+
+QString OpenVpnUiPlugin::tryToCopyToCertificatesDirectory(const QString &connectionName, const QString &sourceFilePath)
+{
+    const QString certificatesDirectory = KStandardDirs::locateLocal("data", "networkmanagement/certificates/");
+    const QString absoluteFilePath = certificatesDirectory + connectionName + '_' + QFileInfo(sourceFilePath).completeBaseName();
+    QFile sourceFile(sourceFilePath);
+
+    QDir().mkpath(certificatesDirectory);
+    if (!sourceFile.copy(absoluteFilePath)) {
+        KMessageBox::information(0, i18n("Error copying file to %1: %2", absoluteFilePath, sourceFile.errorString()));
+        return sourceFilePath;
+    }
+
+    return absoluteFilePath;
+}
+
 bool OpenVpnUiPlugin::exportConnectionSettings(const NetworkManager::ConnectionSettings::Ptr &connection, const QString &fileName)
 {
     QFile expFile(fileName);
@@ -655,8 +787,13 @@ bool OpenVpnUiPlugin::exportConnectionSettings(const NetworkManager::ConnectionS
             }
         }
     }
-    // Export X-NM-Routes
+    // Export never default setting.
     NetworkManager::Ipv4Setting::Ptr ipv4Setting = connection->setting(NetworkManager::Setting::Ipv4).dynamicCast<NetworkManager::Ipv4Setting>();
+    if (!ipv4Setting->neverDefault()) {
+        line = QString(REDIRECT_GATEWAY_TAG) + '\n';
+        expFile.write(line.toLatin1());
+    }
+    // Export X-NM-Routes
     if (!ipv4Setting->routes().isEmpty()) {
         QString routes;
         foreach(const NetworkManager::IpRoute &route, ipv4Setting->routes()) {
